%
% ALPHA_SHAPE calcula el alpha-shape de una nube de puntos
%
%  a_shape = ALPHA_SHAPE(puntos,alfa)
%
function a_shape = alpha_shape(puntos,alfa)

    if nargin < 1
        error('debe introducir una nube de puntos');
    end	

    if nargin < 2
        alfa = 1; %valor por defecto
    end	


    % PASO 1: triangulacin de Delaunay

    triangulos = delaunayn(puntos);
    if size(triangulos,1) == 0
        printf('Los puntos no estn en posicin general. No se puede triangular');
        error;
    end

    %extraemos los segmentos y la matriz de conexin de vrtices de la triangulacin
    [segmentos_delaunay,Cdelaunay] = segmentos_triangulos(triangulos);


    % PASO 2: Generacin del alfa-complex

    incluidos_alfa_complex = zeros(size(segmentos_delaunay,1),1); 

    %aplicamos el alfa-test a cada segmento
    for i = 1:size(segmentos_delaunay,1)
        %condicion C1
        a = segmentos_delaunay(i,1);
        b = segmentos_delaunay(i,2);
        centro_ab = (puntos(a,:) + puntos(b,:))/2;
        radio_t = norm(centro_ab - puntos(a,:));
        if radio_t < alfa
            %calculamos los puntos mas cercanos al segmento
            vecinos_segmento = Cdelaunay(a,:) | Cdelaunay(b,:);
            %nos excluimos a nosotros mismos
            vecinos_segmento(a) = 0;
            vecinos_segmento(b) = 0;
            puntos_cercanos = find(vecinos_segmento==1);
            normas_puntos_cercanos = zeros(length(puntos_cercanos),1);

            %es alfa complex a menos que se demuestre lo contrario
            incluidos_alfa_complex(i) = 1;
            %calculamos la distancias de los puntos ms cercanos y comprobamos si est dentro de la circunferencia
            for j = 1:length(puntos_cercanos)
                normas_puntos_cercanos(j) = norm(centro_ab - puntos(puntos_cercanos(j),:));
                %si hay alguno que cae dentro, no se cumple condicion C1
                if normas_puntos_cercanos(j) < radio_t
                    incluidos_alfa_complex(i) = 0;
                end
            end
            %si no se cumple C1, intentamos ver si se cumple C2
            if incluidos_alfa_complex(i) == 0
                %buscamos el vecino ms cercano
                [minimo,I] = min(normas_puntos_cercanos);
                c = puntos(puntos_cercanos(I),:);
                %calculamos la circunferencia que toca los puntos [a,b,c]
                [centro,radio] = circunferencia3p(puntos(a,:),puntos(b,:),c);
                %el radio de la circusfera tiene que ser menor que alfa
                if radio < alfa
                    incluidos_alfa_complex(i) = 1;
                    %cumple C2 a menos que la circunferencia envuelva a otro punto
                    %TODO Falta comprobar si esta circunferencia engloba a otros
                end
            end
        end
    end

    %almacenamos segmentos que forman parte del alfa-complex
    segmentos_alfa_complex(:,1:2) = segmentos_delaunay(find(incluidos_alfa_complex==1),:);


    % PASO 3: Eliminar segmentos del interior del shape

    %calculamos matriz de conexiones de los segmentos del alfa-complex
    Calfa_complex = conexiones_segmentos(segmentos_alfa_complex);

    %Buscamos qu segmentos son internos, de borde o aislados
    Cshape = clasifica_conexiones(Calfa_complex);

    assert_equal(size(Calfa_complex,1),size(Cshape));


    %calculamos segmentos del alfa-shape
    %buscamos los marcados con 1 o 2, porque los marcados con 3 son segmentos situados
    %dentro del shape
    segmentos_alfa_shape = segmentos_c(Cshape,[1,2]);


    %devolvemos el alpha-shape como una estructura
    a_shape.puntos = puntos;
    a_shape.alfa = alfa;
    a_shape.triangulos_delaunay = triangulos;
    a_shape.segmentos_alfa_complex = segmentos_alfa_complex;
    a_shape.segmentos_delaunay = segmentos_delaunay;
    a_shape.segmentos_alfa_shape = segmentos_alfa_shape;
    a_shape.Cshape = Cshape;
    a_shape.Calfa_complex = Calfa_complex;


return
