%
% CIRCUNFERENCIA3P Circunferencia a partir de 3 puntos
%
%  [centro,radio] = CIRCUNFERENCIA3P(a,b,c) devuelve el centro y 
%    el radio de la circunferencia que incluye a los 3 puntos
%
function [centro,radio] = circunferencia3p(arg1,arg2,arg3)

    if nargin < 3
        error('se requieren 3 puntos como parmetro');
    end	

    %seleccionamos los puntos que estn ms separados entre s
    puntos(1:3,:) = [arg1;arg2;arg3];
    combinaciones = nchoosek([1,2,3],2);
    for i = 1:3
        normas(i) = norm(puntos(combinaciones(i,1),:) - puntos(combinaciones(i,2),:));
    end
    [v,I] = max(normas);
    a = puntos(combinaciones(I,1),:);
    b = puntos(combinaciones(I,2),:);
    c = arg1 + arg2 + arg3 - a - b;

    %precondiciones: ab forman el segmento ms grande
    assert(norm(a-b)>=norm(a-c));
    assert(norm(a-b)>=norm(b-c));

    mu_t = (a+b)/2;
    u = b - a;
    %vector perpendicular al segmento ab
    v = [-u(2),u(1)];

	% f(t) = norm(v*t + mu_t - c) - norm(v*t + mu_t - a)
    a1 = mu_t(1) - c(1);
    a2 = mu_t(2) - c(2);
    b1 = mu_t(1) - a(1);
    b2 = mu_t(2) - a(2);
    f = inline(sprintf('norm([%d,%d]*t + [%d,%d]) - norm([%d,%d]*t + [%d,%d])',v(1),v(2),a1,a2,v(1),v(2),b1,b2));

	% aplicamos Newton-Rhapson para refinar t tal que
	% f(t) = norm(v*t + mu_t - c) - norm(v*t + mu_t - a) = 0
	% ti+1 = ti - f(ti)/f'(ti)
    TOL = 0.00000001;
    MAX_NEWTON_ITER = 50;
    h = 0.001;
    t = 0; 
    niter = 0;
    while abs(f(t)) > TOL && niter < MAX_NEWTON_ITER
        derivada_f = (f(t+h) - f(t-h))/(2*h);    
       % fprintf('t = %d : f(t) = %d\n',t,f(t));
        %corregimos division por 0
        if derivada_f == 0
            derivada_f = TOL;
        end
        t = t - f(t)/derivada_f;
        niter = niter + 1;
    end
    assert(niter < MAX_NEWTON_ITER,'mximo de interaciones rebasado. Posiblemente el mtodo no haya convergido');

    centro = v*t + mu_t;
    radio = norm(centro-a);
   
    assert_float_equal(norm(a-centro),radio,'distancia a-centro dista del radio',0.0001);
    assert_float_equal(norm(b-centro),radio,'distancia b-centro dista del radio',0.0001);
    assert_float_equal(norm(c-centro),radio,'distancia c-centro dista del radio',0.0001);

return
