%
% CLASIFICA_CONEXIONES matriz de conexiones clasificadas
%
%  Calpha_shape = CLASIFICA_CONEXIONES(Calfa_complex) Devuelve la matriz de conexiones
%      con un 1 para los segmentos singulares (no es borde de nadie), con un 2 para los regulares
%      (borde de un simplex de mayor dimensin) y con un 3 para los interiores. Los segmentos
%      interiores no pertenecen al alfa-shape.
%
function Calpha_shape = clasifica_conexiones(Calfa_complex)

    if nargin < 1
        Calpha_complex = 0; %valor por defecto
    end	


    Cshape = triu(Calfa_complex);

    %calculamos pesos de las conexiones en funcin del nmero de triangulos que
    %forma cada segmento. los simples no forman triangulos, los regulares uno y los
    %interiores 2.
    n = size(Calfa_complex,1);
    for i = 1:n
        a = i;
        vecinos_a = find(Cshape(a,:)>0);
        for j = 1:length(vecinos_a)
            b = vecinos_a(j);
            vecinos_b = find(Cshape(b,:)>0);
            for k = 1:length(vecinos_b)
                c = vecinos_b(k);
                %si formamos un triangulo abc, aumentamos peso de los segmentos que lo forman
                if length(find(vecinos_a==c)) == 1
                    Cshape(a,b) = Cshape(a,b) + 1;
                    Cshape(b,c) = Cshape(b,c) + 1;
                    Cshape(a,c) = Cshape(a,c) + 1;
                end
            end
        end
    end

    Calpha_shape = Cshape + Cshape';

return
