%
% SEGMENTOS_C Segmentos matriz de conexiones
%
%  segmentos = SEGMENTOS_C(C) Devuelve los segmentos definidos por la matriz de 
%       conexiones C. La marca utilizada por defecto es 1. C es simtrica
%
%  segmentos = SEGMENTOS_C(C,ID) Donde ID es un vector de los identificadores que se usarn para
%       encontrar los segmentos del tipo que queremos. 
%
%    P.e. , segmentos_c(C,[1,2]) devuelve los segmentos formados por vertices marcados con 1 o 2
%
function segmentos = segmentos_c(C,ID)

    if nargin < 1
        error('debe introducir una matriz de conexiones');
    end	

    if nargin < 2
        ID = [1]; %por defecto, buscamos valores con 1s
    end	

    segmentos = [];
    C = triu(C); %nos quedamos con el triangulo superior para evitar repetidos
    
    %calculamos los segmentos a partir de las conexiones entre vrtices
    n = 0; %numero de segmentos
    for i = 1:size(C,1)
        %buscamos todos los elementos de la fila con los ids elegidos
        conexiones = [];
        for id = 1:length(ID)
            conexiones = [conexiones,find(C(i,:)==ID(id))];
        end
        %guardamos los segmentos
        for j = 1:length(conexiones)
            n = n + 1;
            segmentos(n,1:2) = [i,conexiones(j)];
        end
    end

return
